import { onMounted, ref, watch } from "vue";
import type { ETableEvent, ETableProps } from "../types/table";
import type {
  ETableColumnCtx,
  ETableColumnEditInfo,
  ETableColumnInfo,
} from "../types/table-column";
import { createUuid } from "../tool";
import { ElInput } from "element-plus";

function addInId(list: ETableColumnInfo[]) {
  return list.map((e) => {
    let inId = createUuid();
    e.independentId = inId;
    if (e.columnChildren) {
      e.columnChildren = addInId(e.columnChildren);
    }
    return e;
  });
}

export function useTableAttrs(props: any, tableInstance: any) {
  const tableAttributeProxy = ref<ETableProps>({});
  const tableEventProxy = ref<ETableEvent>({});
  const tableDataProxy = ref<any[]>([]);
  const tableHeadProxy = ref<ETableColumnInfo[]>([]);
  const columnBasicAttrsProxy = ref<ETableColumnCtx>({});
  const editColumnBasicAttrsProxy = ref<Partial<ETableColumnEditInfo>>({});

  function initInfo() {
    console.log("initInfo");
    tableAttributeProxy.value = { ...(props.tableAttribute || {}) };
    tableEventProxy.value = { ...(props.tableEvent || {}) };
    columnBasicAttrsProxy.value = { ...(props.columnBasicAttrs || {}) };
    editColumnBasicAttrsProxy.value = { ...(props.editColumnBasicAttrs || {}) };
    tableDataProxy.value = props.tableData.map((e: Recordable) => ({ ...e }));
    tableHeadProxy.value = props.tableHead.map((e: Recordable) => ({ ...e }));

    if (tableInstance.value) tableInstance.value.doLayout();
  }

  watch(
    props,
    () => {
      initInfo();
    },
    { deep: true }
  );

  onMounted(() => {
    initInfo();
  });

  return {
    tableAttributeProxy,
    tableEventProxy,
    tableDataProxy,
    tableHeadProxy,
    columnBasicAttrsProxy,
    editColumnBasicAttrsProxy,
  };
}

export function useSetStyle(styles: string | string[], tableInstance: any) {
  if (styles) {
    if (typeof styles === "string") {
      useOutStyles(styles, tableInstance.value);
    } else if (styles instanceof Array) {
      styles.forEach((e) => useOutStyles(e, tableInstance.value));
    }
  }
}

export function useOutStyles(s: string, tableInstance: any) {
  let dom = tableInstance._.vnode.el as Element;
  let shadowRoot = dom.parentNode;
  let styles = shadowRoot?.querySelectorAll("style") || [];
  let style = document.createElement("style");
  style.setAttribute("name", "out");
  style.innerHTML = s;
  let hasAdd = false;
  styles.forEach((e) => {
    if (e.innerText === s) {
      hasAdd = true;
    }
  });
  !hasAdd && shadowRoot?.appendChild(style);
}

export function slotComHandler(
  slotItem: {
    component: any;
    attrs?: Recordable;
    event?: Recordable<Function>;
  },
  tableInstance: any
) {
  const Com = slotItem.component;
  useSetStyle(Com.styles, tableInstance);

  return ({ column, $index }: any) => (
    <Com
      {...slotItem.attrs}
      {...slotItem.event}
      column={column}
      index={$index}
      inputEl={ElInput}
    />
  );
}
