import type { TableProps } from "element-plus/lib/components/table/src/table/defaults";
import type {
  ETableColumnCtx,
  ETableColumnInfo,
  ETableColumnEditInfo,
} from "./table-column";

export type ETableProps = Partial<TableProps<any>>;
type row = Recordable;

export interface sortParams {
  column: ETableColumnCtx;
  prop: string;
  order: string;
}

export interface slotParams {
  column?: ETableColumnCtx;
  row?: Recordable | undefined;
  $index?: number;
  expanded?: boolean;
  store?: Recordable;
}

export interface ETableEvent {
  select?: ((selection: boolean, row: row) => any) | undefined;
  "expand-change"?: ((row: row, t: row[] | boolean) => any) | undefined;
  "current-change"?: ((row: row, oldRow: row) => any) | undefined;
  "select-all"?: ((selection: row[]) => any) | undefined;
  "selection-change"?: ((selection: row[]) => any) | undefined;
  "cell-mouse-enter"?:
    | ((
        row: row,
        column: ETableColumnCtx,
        cell: HTMLElement,
        event: MouseEvent
      ) => any)
    | undefined;
  "cell-mouse-leave"?:
    | ((
        row: row,
        column: ETableColumnCtx,
        cell: HTMLElement,
        event: MouseEvent
      ) => any)
    | undefined;
  "cell-contextmenu"?:
    | ((
        row: any,
        column: ETableColumnCtx,
        cell: HTMLElement,
        event: PointerEvent
      ) => any)
    | undefined;
  "cell-click"?:
    | ((
        row: row,
        column: ETableColumnCtx,
        cell: HTMLElement,
        event: PointerEvent
      ) => any)
    | undefined;
  "cell-dblclick"?:
    | ((
        row: row,
        column: ETableColumnCtx,
        cell: HTMLElement,
        event: MouseEvent
      ) => any)
    | undefined;
  "row-click"?:
    | ((row: row, column: ETableColumnCtx, event: PointerEvent) => any)
    | undefined;
  "row-contextmenu"?:
    | ((row: row, column: ETableColumnCtx, event: PointerEvent) => any)
    | undefined;
  "row-dblclick"?:
    | ((row: row, column: ETableColumnCtx, event: MouseEvent) => any)
    | undefined;
  "header-click"?:
    | ((column: ETableColumnCtx, event: PointerEvent) => any)
    | undefined;
  "header-contextmenu"?:
    | ((column: ETableColumnCtx, event: PointerEvent) => any)
    | undefined;
  "sort-change"?: (({ column, prop, order }: sortParams) => any) | undefined;
  "filter-change"?: ((filters: string) => any) | undefined;
  "header-dragend"?:
    | ((
        newWidth: number,
        oldWidth: number,
        column: ETableColumnCtx,
        event: MouseEvent
      ) => any)
    | undefined;
}

export interface editUpdateArguments {
  value: any;
  originalValue: string | any;
  showValue: string;
  index: number;
  row: Recordable;
  column: ETableColumnCtx;
  [k: string]: any;
}

export interface EditHandlerType {
  editUpdate: ({
    value,
    originalValue,
    showValue,
    index,
    row,
    column,
  }: Partial<editUpdateArguments>) => void;
}

export interface tableComponentsProps {
  tableAttribute?: ETableProps;
  tableEvent?: ETableEvent;
  tableData?: any[];
  tableHead?: ETableColumnInfo[];
  columnBasicAttrs?: ETableColumnCtx;
  editColumnBasicAttrs?: Partial<ETableColumnEditInfo>;
}
